package xml;

import java.io.File;
import java.io.IOException;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.w3c.dom.Document;
import org.xml.sax.SAXParseException;

public class XMLValidator
{
    private final static int INVALID_PARAMETERS = 1;
    private final static int INVALID_XML = 2;
    private final static int INVALID_FILE = 3;
    private final static int ANY_ERROR = 4;

    public static void main(String[] args)
    {
        if (args.length != 2)
        {
            System.out.println("Usage: XMLValidator PathToXMLFile PathToSchemaFile");
            System.exit(INVALID_PARAMETERS);
        }

        String xmlFile = args[0];
        String xsdFile = args[1];

        try
        {
            validateXml(xmlFile, xsdFile);
        }
        catch (Exception e)
        {
            System.out.println("Error: " + e.getMessage());
            System.exit(ANY_ERROR);
        }
    }

    private static void validateXml(String xmlFile, String xsdFile) throws Exception
    {
        // create a factory that understands namespaces and validates the XML input
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);

        try
        {
            // read the XML file
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document mDoc = builder.parse(new File(xmlFile));

            // create a SchemaFactory capable of understanding schemas
            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

            // load the schema
            Schema schema = schemaFactory.newSchema(new File(xsdFile));

            // validate the XML file
            Validator validator = schema.newValidator();
            validator.validate(new DOMSource(mDoc));
        }
        catch (SAXParseException spe)
        {
            // error generated by the parser
            System.out.println("Parser error: URI " + spe.getSystemId() + ", Line " + spe.getLineNumber() + ", Message: " + spe.getMessage());
            System.exit(INVALID_XML);
        }
        catch (IOException ioe)
        {
            System.out.println("Error reading file '" + xmlFile + "'.");
            System.exit(INVALID_FILE);
        }
    }
}
